@@ -82,6 +82,7 @@ rc <- function(tab, nd=1, symmetric=FALSE, diagonal=FALSE,
   newclasses <- if(symmetric) c("rc.symm", "rc", "assocmod") else c("rc", "assocmod")
   class(model) <- c(newclasses, class(model))
 
+  model$call.gnm <- model$call
   model$call <- match.call()
 
   model$assoc <- assoc(model, weighting=weighting, rowsup=rowsup, colsup=colsup)
@@ -117,7 +118,7 @@ assoc.rc <- function(model, weighting=c("marginal", "uniform", "none"),
   tab <- prepareTable(model$data, TRUE, rowsup, colsup)
   vars <- names(dimnames(tab))
 
-  # Weight with marginal frequencies, cf. Becker & Clogg (1994), p. 83-84, and Becker & Clogg (1989), p. 144.
+  # Weight with marginal frequencies, cf. Clogg & Shihadeh (1994), p. 83-84, and Becker & Clogg (1989), p. 144.
   weighting <- match.arg(weighting)
   if(weighting == "marginal") {
       rp <- prop.table(apply(tab, 1, sum, na.rm=TRUE))
@@ -238,7 +239,8 @@ assoc.rc <- function(model, weighting=c("marginal", "uniform", "none"),
   }
 
   obj <- list(phi = phi, row = row, col = col, diagonal = dg,
-              weighting = weighting, row.weights = row.weights, col.weights = col.weights)
+              weighting = weighting, row.weights = row.weights, col.weights = col.weights,
+              vars = vars)
 
 
   ## Supplementary rows/columns
@@ -264,7 +266,7 @@ assoc.rc.symm <- function(model, weighting=c("marginal", "uniform", "none"),
   tab <- prepareTable(model$data, TRUE, rowsup, colsup)
   vars <- names(dimnames(tab))
 
-  # Weight with marginal frequencies, cf. Becker & Clogg (1994), p. 83-84, and Becker & Clogg (1989), p. 144.
+  # Weight with marginal frequencies, cf. Clogg & Shihadeh (1994), p. 83-84, and Becker & Clogg (1989), p. 144.
   weighting <- match.arg(weighting)
   if(weighting == "marginal")
       p <- prop.table(apply(tab, 1, sum, na.rm=TRUE) + apply(tab, 2, sum, na.rm=TRUE))
@@ -361,7 +363,8 @@ assoc.rc.symm <- function(model, weighting=c("marginal", "uniform", "none"),
   }
 
   obj <- list(phi = phi, row = sc, col= sc, diagonal = dg,
-              weighting = weighting, row.weights = row.weights, col.weights = col.weights)
+              weighting = weighting, row.weights = row.weights, col.weights = col.weights,
+              vars = vars)
 
   ## Supplementary rows/columns
   if(!is.null(rowsup) || !is.null(colsup)) {
