@@ -149,6 +149,7 @@ unidiff <- function(tab, diagonal=c("included", "excluded", "only"),
 
   class(model) <- c("unidiff", class(model))
 
+  model$call.gnm <- model$call
   model$call <- match.call()
 
   model
@@ -212,6 +213,8 @@ summary.unidiff <- function(object, ...) {
       colnames(interaction) <- c("Estimate", "Std. Error", "Pr(>|z|)")
   }
 
+  summ.gnm <- summary(asGnm(object))
+
   res <- list(call=object$call,
               deviance.resid=residuals(object, type="deviance"),
               chisq=sum(na.omit(c(residuals(object, "pearson")^2))),
@@ -221,7 +224,9 @@ summary.unidiff <- function(object, ...) {
               weighting=object$unidiff$weighting,
               deviance=object$deviance, df.residual=object$df.residual,
               bic=object$deviance - log(sum(na.omit(c(object$data)))) * object$df.residual,
-              aic=object$deviance - 2 * object$df.residual)
+              aic=object$deviance - 2 * object$df.residual,
+              # Needed by R functions like anova()
+              family=object$family, dispersion=summ.gnm$dispersion, df=summ.gnm$df)
 
   class(res) <- "summary.unidiff"
 
